@@ -25,7 +25,7 @@ from monai.data.dataset import CacheDataset, Dataset, PersistentDataset
 from monai.transforms import Compose
 
 
-_SlidesOrTilesDataset = TypeVar('_SlidesOrTilesDataset', SlidesDataset, TilesDataset)
+_SlidesOrTilesDataset = TypeVar("_SlidesOrTilesDataset", SlidesDataset, TilesDataset)
 
 
 class CacheMode(Enum):
@@ -57,6 +57,7 @@ class HistoDataModule(LightningDataModule, Generic[_SlidesOrTilesDataset]):
         pl_replace_sampler_ddp: bool = True,
         dataloader_kwargs: Optional[Dict[str, Any]] = None,
         dataframe_kwargs: Optional[Dict[str, Any]] = None,
+        class_weights: Optional[torch.Tensor] = None,
     ) -> None:
         """
         :param root_path: Root directory of the source dataset.
@@ -80,6 +81,7 @@ class HistoDataModule(LightningDataModule, Generic[_SlidesOrTilesDataset]):
         :param pl_replace_sampler_ddp: If True, replace the sampler with a DistributedSampler when using DDP.
         :param dataloader_kwargs: Additional keyword arguments for the training, validation, and test dataloaders.
         :param dataframe_kwargs: Keyword arguments to pass to `pd.read_csv()` when loading the dataset CSV.
+        :param class_weights: Class weights to use for the dataset. If None, will compute them from the dataset.
         """
 
         batch_size_inf = batch_size_inf or batch_size
@@ -97,7 +99,7 @@ class HistoDataModule(LightningDataModule, Generic[_SlidesOrTilesDataset]):
         self.test_dataset: _SlidesOrTilesDataset
         self.dataframe_kwargs = dataframe_kwargs or {}
         self.train_dataset, self.val_dataset, self.test_dataset = self.get_splits()
-        self.class_weights = self.train_dataset.get_class_weights()
+        self.class_weights = class_weights if class_weights is not None else self.train_dataset.get_class_weights()
         self.seed = seed
         self.dataloader_kwargs = dataloader_kwargs or {}
 
@@ -118,29 +120,28 @@ class HistoDataModule(LightningDataModule, Generic[_SlidesOrTilesDataset]):
                 logging.info("pl_replace_sampler_ddp is False, setting DistributedSampler for training dataloader.")
                 return DistributedSampler(dataset, shuffle=True, seed=self.seed)
             else:
-                logging.info("pl_replace_sampler_ddp is False, setting UnrepeatedDistributedSampler for validation and "
-                             "test dataloaders. This will ensure that each process gets a unique set of samples. "
-                             "If you want to use DistributedSampler, set pl_replace_sampler_ddp to True.")
+                logging.info(
+                    "pl_replace_sampler_ddp is False, setting UnrepeatedDistributedSampler for validation and "
+                    "test dataloaders. This will ensure that each process gets a unique set of samples. "
+                    "If you want to use DistributedSampler, set pl_replace_sampler_ddp to True."
+                )
                 return UnrepeatedDistributedSampler(dataset, shuffle=False, seed=self.seed)
         return None
 
     def train_dataloader(self) -> DataLoader:
-        return self._get_dataloader(self.train_dataset,  # type: ignore
-                                    shuffle=True,
-                                    stage=ModelKey.TRAIN,
-                                    **self.dataloader_kwargs)
+        return self._get_dataloader(
+            self.train_dataset, shuffle=True, stage=ModelKey.TRAIN, **self.dataloader_kwargs  # type: ignore
+        )
 
     def val_dataloader(self) -> DataLoader:
-        return self._get_dataloader(self.val_dataset,  # type: ignore
-                                    shuffle=False,
-                                    stage=ModelKey.VAL,
-                                    **self.dataloader_kwargs)
+        return self._get_dataloader(
+            self.val_dataset, shuffle=False, stage=ModelKey.VAL, **self.dataloader_kwargs  # type: ignore
+        )
 
     def test_dataloader(self) -> DataLoader:
-        return self._get_dataloader(self.test_dataset,  # type: ignore
-                                    shuffle=False,
-                                    stage=ModelKey.TEST,
-                                    **self.dataloader_kwargs)
+        return self._get_dataloader(
+            self.test_dataset, shuffle=False, stage=ModelKey.TEST, **self.dataloader_kwargs  # type: ignore
+        )
 
 
 class TilesDataModule(HistoDataModule[TilesDataset]):
@@ -254,8 +255,9 @@ class TilesDataModule(HistoDataModule[TilesDataset]):
 
         return transformed_bag_dataset
 
-    def _get_dataloader(self, dataset: TilesDataset, stage: ModelKey, shuffle: bool,
-                        **dataloader_kwargs: Any) -> DataLoader:
+    def _get_dataloader(
+        self, dataset: TilesDataset, stage: ModelKey, shuffle: bool, **dataloader_kwargs: Any
+    ) -> DataLoader:
         transformed_bag_dataset = self._load_dataset(dataset, stage=stage, shuffle=shuffle)
         bag_dataset: BagDataset = transformed_bag_dataset.data  # type: ignore
         generator = bag_dataset.bag_sampler.generator
@@ -305,8 +307,9 @@ class SlidesDataModule(HistoDataModule[SlidesDataset]):
         transforms.set_random_state(seed=self.seed)
         return Dataset(slides_dataset, transforms)
 
-    def _get_dataloader(self, dataset: SlidesDataset, stage: ModelKey, shuffle: bool,
-                        **dataloader_kwargs: Any) -> DataLoader:
+    def _get_dataloader(
+        self, dataset: SlidesDataset, stage: ModelKey, shuffle: bool, **dataloader_kwargs: Any
+    ) -> DataLoader:
         transformed_slides_dataset = self._load_dataset(dataset, stage)
         generator = _create_generator(self.seed)
         sampler = self._get_ddp_sampler(transformed_slides_dataset, stage)
