@@ -20,7 +20,7 @@ sys.path.insert(0, str(health_azure_root))
 
 from health_ml.utils.common_utils import df_to_json  # noqa: E402
 from health_azure import aggregate_hyperdrive_metrics  # NOQA: E402
-from health_azure.utils import get_aml_run_from_run_id, get_metrics_for_childless_run  # NOQA: E402
+from health_azure.utils import get_aml_run_from_run_id, get_metrics_for_run  # NOQA: E402
 
 
 def print_metrics(metrics_list: List[str], metrics_df: pd.DataFrame) -> None:
@@ -55,13 +55,10 @@ def get_metrics_from_run(run: Run, metrics_list: List[str]) -> pd.DataFrame:
         raise ValueError("Can't get metrics for an OfflineRun")
     if len(list(run.get_children())) > 0:
         metrics_df = aggregate_hyperdrive_metrics(
-            child_run_arg_name="crossval_index",
-            run=run,
-            keep_metrics=metrics_list)
+            child_run_arg_name="crossval_index", run=run, keep_metrics=metrics_list
+        )
     else:
-        metrics_df = get_metrics_for_childless_run(
-            run=run,
-            keep_metrics=metrics_list)
+        metrics_df = get_metrics_for_run(run=run, keep_metrics=metrics_list)
     print_metrics(metrics_list, metrics_df)
     return metrics_df
 
@@ -87,10 +84,15 @@ def upload_regression_metrics_file_to_run(metrics_df: pd.DataFrame, run: Run) ->
 if __name__ == "__main__":
     parser = ArgumentParser()
     parser.add_argument("--run", type=str, default='', help="The run id to retrieve metrics from")
-    parser.add_argument("--metrics_list", type=str,
-                        help="A comma-separated list of metrics names to retrieve from the AML Run")
-    parser.add_argument("--upload_metrics_file", type=bool, default=True,
-                        help="If True, saves a json file of the metrics dataframe and uploads this to the AML Run")
+    parser.add_argument(
+        "--metrics_list", type=str, help="A comma-separated list of metrics names to retrieve from the AML Run"
+    )
+    parser.add_argument(
+        "--upload_metrics_file",
+        type=bool,
+        default=True,
+        help="If True, saves a json file of the metrics dataframe and uploads this to the AML Run",
+    )
     args = parser.parse_args(sys.argv[1:])
     run_id = args.run
     metrics_list = args.metrics_list.split(",")
