@@ -22,6 +22,9 @@ from health_cpath.models.deepmil import DeepMILModule
 from health_cpath.utils.naming import ModelKey, ResultsKey
 from health_cpath.utils.output_utils import BatchResultsType
 
+
+logger = logging.getLogger(__name__)
+
 LossCacheDictType = Dict[Union[ResultsKey, str], List]
 LossDictType = Dict[str, List]
 AnomalyDictType = Dict[ModelKey, List[str]]
@@ -243,7 +246,7 @@ class LossAnalysisCallback(Callback):
         epoch: int,
         stage: ModelKey,
         high: Optional[bool] = None,
-        num_values: Optional[int] = None
+        num_values: Optional[int] = None,
     ) -> List[np.ndarray]:
         """Selects the values corresponding to keys from a dataframe for the given epoch and stage.
 
@@ -349,12 +352,12 @@ class LossAnalysisCallback(Callback):
         for slide_id, loss in loss_values_copy.items():
             try:
                 if np.isnan(loss).any():
-                    logging.warning(f"NaNs found in loss values for slide {slide_id}.")
+                    logger.warning(f"NaNs found in loss values for slide {slide_id}.")
                     self.nan_slides[stage].append(slide_id)
                     loss_values.pop(slide_id)
             except Exception as e:
-                logging.warning(f"Error while checking for NaNs in loss values for slide {slide_id} with error {e}.")
-                logging.warning(f"Loss values that caused the issue: {loss}")
+                logger.warning(f"Error while checking for NaNs in loss values for slide {slide_id} with error {e}.")
+                logger.warning(f"Loss values that caused the issue: {loss}")
                 self.anomaly_slides[stage].append(slide_id)
                 loss_values.pop(slide_id)
         self.save_slide_ids(self.nan_slides[stage], self.get_nan_slides_file(stage))
@@ -420,7 +423,7 @@ class LossAnalysisCallback(Callback):
         epoch: int,
         stage: ModelKey,
         high: bool,
-        figsize: Tuple[float, float] = (15, 15)
+        figsize: Tuple[float, float] = (15, 15),
     ) -> None:
         """Plots the loss values for each slide across all epochs in a heatmap.
 
@@ -484,11 +487,11 @@ class LossAnalysisCallback(Callback):
         for epoch in self.epochs_range:
             epoch_slides = self.select_slides_for_epoch(epoch, stage=stage)
 
-            top_slides = epoch_slides[:self.num_slides_heatmap]
+            top_slides = epoch_slides[: self.num_slides_heatmap]
             top_slides_loss_values = self.select_all_losses_for_selected_slides(top_slides, stage=stage)
             self.plot_loss_heatmap_for_slides_of_epoch(top_slides_loss_values, epoch, stage, high=True)
 
-            bottom_slides = epoch_slides[-self.num_slides_heatmap:]
+            bottom_slides = epoch_slides[-self.num_slides_heatmap :]
             bottom_slides_loss_values = self.select_all_losses_for_selected_slides(bottom_slides, stage=stage)
             self.plot_loss_heatmap_for_slides_of_epoch(bottom_slides_loss_values, epoch, stage, high=False)
         self.loss_cache[stage] = self.get_empty_loss_cache()  # reset loss cache
@@ -498,7 +501,7 @@ class LossAnalysisCallback(Callback):
         if self.log_exceptions:
             # If something goes wrong, we don't want to crash the training. We just log the error and carry on
             # validation.
-            logging.warning(f"Error while detecting {stage} loss values outliers: {exception}")
+            logger.warning(f"Error while detecting {stage} loss values outliers: {exception}")
         else:
             # If we want to debug the error, we raise it. This will crash the training. This is useful when
             # running smoke tests.
@@ -511,7 +514,7 @@ class LossAnalysisCallback(Callback):
         outputs: BatchResultsType,
         batch: Dict,
         batch_idx: int,
-        unused: int = 0
+        unused: int = 0,
     ) -> None:
         """Caches train loss values per slide at each training step in a local variable self.loss_cache."""
         self.update_loss_cache(trainer, outputs, batch, stage=ModelKey.TRAIN)
@@ -523,7 +526,7 @@ class LossAnalysisCallback(Callback):
         outputs: BatchResultsType,
         batch: Any,
         batch_idx: int,
-        dataloader_idx: int
+        dataloader_idx: int,
     ) -> None:
         """Caches validation loss values per slide at each training step in a local variable self.loss_cache."""
         self.update_loss_cache(trainer, outputs, batch, stage=ModelKey.VAL)
